@@ -206,7 +206,7 @@ module REXML
 		end
 
 		def Declaration.parse_source source, listener
-			md = src.match( pattern, true )
+			md = source.match( pattern, true )
 			listener.send inspect.downcase, md[1]
 		end
 	end
@@ -218,6 +218,11 @@ module REXML
 		def pattern
 			PATTERN_RE
 		end
+		def AttlistDecl.parse_source source, listener
+			md = source.match( PATTERN_RE, true )
+			#listener.send inspect.downcase, md[1]
+			listener.attlistdecl md[1]
+		end
 	end
 
 	class ElementDecl < Declaration
@@ -227,6 +232,11 @@ module REXML
 		def pattern
 			PATTERN_RE
 		end
+		def ElementDecl.parse_source source, listener
+			md = source.match( PATTERN_RE, true )
+			#listener.send inspect.downcase, md[1]
+			listener.elementdecl md[1]
+		end
 	end
 
 	class EntityDecl < Child
@@ -278,7 +288,8 @@ module REXML
 		def EntityDecl.parse_source source, listener
 			md = source.match( PATTERN_RE, true )
 			thing = md[0].squeeze " \t\n\r"
-			listener.send inspect.downcase, thing 
+			#listener.send inspect.downcase, thing 
+			listener.entitydecl thing 
 		end
 	end
 
@@ -314,7 +325,8 @@ module REXML
 		def NotationDecl.parse_source source, listener
 			md = source.match( PATTERN_RE, true )
 			thing = md[0].squeeze " \t\n\r"
-			listener.send inspect.downcase, thing 
+			#listener.send inspect.downcase, thing 
+		    listener.notationdecl thing
 		end
 	end
 end
