@@ -5,11 +5,11 @@ module REXML
 	# superclasses of other objects.  Nodes have siblings.
 	module Node
 		# The order in which the substitutions occur
-		SPECIALS = [ /&(?!#?[\w-]+;)/u, /</u, />/u, /"/u, /'/u ]
-		SLAICEPS = [ "&", "<", ">", '"', "'" ]
+		SPECIALS = [ /&(?!#?[\w-]+;)/u, /</u, />/u, /"/u, /'/u, /\r/u ]
+		SUBSTITUTES = ['&amp;', '&lt;', '&gt;', '&quot;', '&apos;', '&#13;']
 		# Characters which are substituted in written strings
-		SUBSTITUTES = ['&amp;', '&lt;', '&gt;', '&quot;', '&apos;']
-		SETUTITSBUS = [/&amp;/u, /&lt;/u, /&gt;/u, /&quot;/u, /&apos;/u]
+		SLAICEPS = [ '<', '>', '"', "'", '&' ]
+		SETUTITSBUS = [ /&lt;/u, /&gt;/u, /&quot;/u, /&apos;/u, /&amp;/u ]
 
 		# @return the next sibling (nil if unset)
 		def next_sibling_node
@@ -57,22 +57,32 @@ module REXML
 			copy.gsub!( SPECIALS[2], SUBSTITUTES[2] )
 			copy.gsub!( SPECIALS[3], SUBSTITUTES[3] )
 			copy.gsub!( SPECIALS[4], SUBSTITUTES[4] )
+			copy.gsub!( SPECIALS[5], SUBSTITUTES[5] )
 			out << copy
 		end
 
 		# Reads text, substituting entities
-		def read_with_substitution( input, illegal=nil )
+		def Node::read_with_substitution( input, illegal=nil )
 			copy = input.clone
 
 			if copy =~ illegal
 				raise ParseException.new( "malformed text: Illegal character #$& in \"#{copy}\"" )
 			end if illegal
 			
-			copy.gsub!( SETUTITSBUS[0], SLAICEPS[0] )
-			copy.gsub!( SETUTITSBUS[1], SLAICEPS[1] )
-			copy.gsub!( SETUTITSBUS[2], SLAICEPS[2] )
-			copy.gsub!( SETUTITSBUS[3], SLAICEPS[3] )
-			copy.gsub!( SETUTITSBUS[4], SLAICEPS[4] )
+			copy.gsub!( /\r\n?/, "\n" )
+			if copy.include? ?&
+				copy.gsub!( SETUTITSBUS[0], SLAICEPS[0] )
+				copy.gsub!( SETUTITSBUS[1], SLAICEPS[1] )
+				copy.gsub!( SETUTITSBUS[2], SLAICEPS[2] )
+				copy.gsub!( SETUTITSBUS[3], SLAICEPS[3] )
+				copy.gsub!( SETUTITSBUS[4], SLAICEPS[4] )
+				copy.gsub!( /&#0*((?:\d+)|(?:x[a-f0-9]+));/ ) {|m|
+					m=$1
+					#m='0' if m==''
+					m = "0#{m}" if m[0] == ?x
+					[Integer(m)].pack('U*')
+				}
+			end
 
 			copy
 		end
